from notecoin.huobi.client import (AccountClient, GenericClient, MarketClient,
                                   TradeClient)
from notecoin.huobi.constant import OrderType
from notetool.tool.secret import read_secret


class TradeOrder:
    def __init__(self):
        self.trade: TradeClient = None
        self.market: MarketClient = None
        self.generic: GenericClient = None
        self.account: AccountClient = None
        self.account_id = None

        self.init()

    def init(self):
        api_key = read_secret(cate1='coin', cate2='huobi', cate3='api_key')
        secret_key = read_secret(cate1='coin', cate2='huobi', cate3='secret_key')
        self.market = MarketClient(api_key=api_key, secret_key=secret_key)
        self.generic = GenericClient(api_key=api_key, secret_key=secret_key)
        self.trade = TradeClient(api_key=api_key, secret_key=secret_key)

        self.account = AccountClient(api_key=api_key, secret_key=secret_key)

        account_dict = self.account.get_accounts()
        # print(account_dict.dict_data['data'][0]['id'])
        self.account_id = account_dict.dict_data['data'][0]['id']

    def buy(self, symbol, amount):
        source = 'spot-api'
        return self.trade.create_order(symbol=symbol,
                                       account_id=self.account_id,
                                       order_type=OrderType.BUY_MARKET,
                                       amount=amount,
                                       source=source,
                                       price=0)





import os

from notebuild.core.core import command_line_parser
from notebuild.manage import BaseServer, ServerManage

# lsof -t -i:8444
# sudo kill -9 `sudo lsof -t -i:8444`
# nohup python notecoin_server.py  >>/notechats/logs/notecoin/strategy-$(date +%Y-%m-%d).log 2>&1 &


class CoinServer(BaseServer):
    def __init__(self):
        path = os.path.abspath(os.path.dirname(__file__))
        super(CoinServer, self).__init__('notecoin_server', path)

    def init(self):
        manage = ServerManage()
        try:
            manage.init()
        except Exception as e:
            print(e)

        manage.add_job(server_name='notecoin_server',
                       directory=self.current_path,
                       command=f"uvicorn notecoin_server:app --host '0.0.0.0' --port 8444 --reload",
                       user='bingtao',
                       stdout_logfile="/notechats/logs/notecoin/strategy.log")
        manage.start()


def notecoin():
    args = command_line_parser()
    package = CoinServer()
    if args.command == 'init':
        package.init()
    elif args.command == 'stop':
        package.stop()
    elif args.command == 'start':
        package.start()
    elif args.command == 'restart':
        package.restart()
    elif args.command == 'help':
        info = """
init
stop
start
restart
        """
        print(info)



from notecoin.okex.websocket.utils import (check, get_local_timestamp, partial,
                                           update_asks, update_bids)


class BaseHandle:
    def __init__(self, channels, *args, **kwargs):
        self.channels = channels

    def solve(self, data) -> bool:
        data = eval(data)
        print(f"{get_local_timestamp()}\t{len(data)}")
        if 'event' in data:
            return False
        channel = data['args']['channel']
        if channel in self.channels:
            return self.handle(data)

    def handle(self, data) -> bool:
        pass


class PublicBooks(BaseHandle):
    def __init__(self, *args, **kwargs):
        super(PublicBooks, self).__init__(channels=['books'], *args, **kwargs)

    def handle(self, data) -> bool:
        l = []
        if data['action'] == 'snapshot':
            for m in l:
                if data['arg']['instId'] == m['instrument_id']:
                    l.remove(m)
            # 获取首次全量深度数据
            bids_p, asks_p, instrument_id = partial(data)
            d = {}
            d['instrument_id'] = instrument_id
            d['bids_p'] = bids_p
            d['asks_p'] = asks_p
            l.append(d)

            # 校验checksum
            checksum = data['data'][0]['checksum']
            # print('推送数据的checksum为:' + str(checksum))
            check_num = check(bids_p, asks_p)
            # print('校验后的checksum为:' + str(check_num))
            if check_num == checksum:
                print("校验结果为:True")
            else:
                print("校验结果为:False，正在重新订阅……")
                return False

        elif data['action'] == 'update':
            for j in l:
                if data['arg']['instId'] == j['instrument_id']:
                    # 获取全量数据
                    bids_p = j['bids_p']
                    asks_p = j['asks_p']
                    # 获取合并后数据
                    bids_p = update_bids(data, bids_p)
                    asks_p = update_asks(data, asks_p)

                    # 校验checksum
                    checksum = data['data'][0]['checksum']
                    # print('推送数据的checksum为:' + str(checksum))
                    check_num = check(bids_p, asks_p)
                    # print('校验后的checksum为:' + str(check_num))
                    if check_num == checksum:
                        print("校验结果为:True")
                    else:
                        print("校验结果为:False，正在重新订阅……")
                        return False
        return True



from fastapi import FastAPI
from notecoin.okex.server.const import account_account, market_tickers
from notecoin.strategy.sell_strategy import AutoSeller

seller = AutoSeller()

app = FastAPI()
app.include_router(account_account)
app.include_router(market_tickers)
app.include_router(seller)

# uvicorn notecoin_server:app --host '0.0.0.0' --port 8444 --reload
